import api from '../api'
import {
	storage,
	navigator,
	indicator,
	modal
} from '../utils/uniApi.js'
import bus from '../utils/eventBus.js'
import tenant from '../tenant/config'

const device = uni.getSystemInfoSync()
const brand = device.brand && device.brand.toLowerCase()
if (brand === 'iphone') {
	var reg = new RegExp('[^\\<\\>]+(?=\\>)', 'g')
	device.model = device.model.match(reg)[0]
}

// #ifdef MP-WEIXIN
const appTopButton = uni.getMenuButtonBoundingClientRect() // 小程序顶部胶囊按钮
// #endif

const common = {
	state: {
		token: storage.get('token') || '',
		permission: storage.get('permission') || {},
		enterPriseType: '',
		UUID: '',
		userInfo: storage.get('userInfo') || {},
		city: {},
		systemInfo: {},
		gps: '',
		platform: device.platform === 'android' ? 'android' : 'ios',
		device: (brand === 'iphone' || brand === 'devtools') ? device.model : brand ? brand + ' ' + device.model :
			device.model,
		// #ifdef MP-WEIXIN
		wrapHeight: (appTopButton.top + 32 + 10) + 'px',
		appTopButton,
		// #endif
		authInfo: {
			openId: '',
			unionId: '',
			sessionKey: '',
			isBind: false
		},
		approve: false, // 是否审核完成
		tenant
	},
	mutations: {
		setToken(state, token) {
			if (token) {
				state.token = token
				storage.set('token', token)
			} else {
				let storageToken = storage.get('token') || ''
				state.token = storageToken
			}
			bus.$emit('setToken')
		},
		setPermission(state, permission) {
			if (permission) {
				state.permission = permission
				storage.set('permission', permission)
			} else {
				let storagePermission = storage.get('permission') || ''
				state.permission = storagePermission
			}
		},
		setEnterpriseType(state, enterPriseType) {
			// console.log(enterPriseType)
			if (enterPriseType) {
				state.enterPriseType = enterPriseType
				storage.set('enterPriseType', enterPriseType)
			} else {
				state.enterPriseType = storage.get('enterPriseType') || ''
			}
		},
		deleteToken(state) {
			state.token = ''
			storage.remove('token')
		},
		setGps(state, gps) {
			state.gps = gps
		},
		setUUID(state) {
			let UUID = storage.get('uuid')
			if (!UUID) {
				UUID = uuid()
				storage.set('uuid', UUID)
			}
			state.UUID = UUID
		},
		setUserInfo(state, userInfo) {
			storage.set('userInfo', userInfo)
			state.userInfo = userInfo
		},
		setCity(state, city) {
			storage.set('selectCity', city)
			state.city = city
		},
		setSystemInfo(state, device) {
			state.device = device
		},
		setApprove(state, value) {
			state.approve = value
		},
		setAuthInfo(state, authInfo) {
			state.authInfo = authInfo
		}
	},
	actions: {
		async init({
			commit,
			dispatch
		}) {
			// if (storage.get('token')) return
			// 登录
			// storage.set('hideLogin', true)
			let l = await new Promise((resolve, reject) => {
				uni.login({
					success: r => resolve(r),
					fail: e => reject(e)
				})
			})
			// 获取code
			api.commonApi.wxLogin({
				code: l.code
			}).then(data => {
				commit('setAuthInfo', {
					openId: data.openId,
					sessionKey: data.sessionKey
				})
				storage.set('authInfo', {
					openId: data.openId,
					sessionKey: data.sessionKey
				})
				if (data.access_token) {
					commit('setToken', data.access_token)
					commit('setEnterpriseType', data.enterprise_type)
					dispatch('getUserInfo')
					let url = storage.get('urlParams') ? JSON.parse(storage.get('urlParams')) : ''
					// console.log(url.path)
					if (url && url.path === 'pages/login/index') {
						navigator.switchTab('/pages/home/index')
					}
				} else {
					storage.clear()
					navigator.login()
				}
				bus.$emit('hideLoading') // 关闭loading
				// storage.set('hideLogin', false) // 显示登录页面
			})
		},
		approve({commit}) { // 小程序审核参数
			api.paramsApi.getParams('mpapprove').then(res => {
				if (res.success) {
					commit('setApprove', res.data.paramValue > 0) // 0 审核中，1 审核完成
				}
			})
		},
		checkUser({
			commit,
			dispatch
		}) { // 判断用户是否注册
			api.commonApi.checkUser().then(res => {
				if (res.success) {
					dispatch('getUserInfo')
				} else {
					uni.showModal({
						// 弹窗提示
						title: '登录',
						content: '您还未登录，无法获取您的用户信息，请先登录',
						confirmText: '去登录',
						cancelText: '暂不登录',
						success(res) {
							if (res.confirm) {
								navigator.replace('/mine/register/index')
							}
						}
					})
				}
			}).finally(() => {
				indicator.close()
			})
		},
		getUserInfo({
			commit
		}) { // 获取用户信息
			let storageToken = storage.get('token') || ''
			if (!storageToken) return // 不强制登录，需要强制登录就删除此行
			return new Promise((resolve, reject) => {
				api.commonApi.getBaseInfo().then(res => {
					if (res.success) {
						commit('setUserInfo', res.data)
						commit('setToken', storageToken)
						// 设置角标文本--【我的】
						// if (res.data.myNew) {
						//   uni.showTabBarRedDot({
						//     index: 1
						//   })
						// } else {
						//   // 取消角标
						//   uni.hideTabBarRedDot({
						//     index: 1
						//   })
						// }
					}
					resolve(res.success)
				})
			})
		},
		getGps({
			commit,
			dispatch
		}, type = 0) {
			// type = 0 只获取gps，type = 1 获取社区信息，
			return new Promise((resolve, reject) => {
				uni.getLocation({
					type: 'gcj02',
					isHighAccuracy: true,
					success(gps) {
						// console.log(gps)
						commit('setGps', gps)
						if (type === 0) {
							dispatch('getCommunity')
						}
						resolve(gps)
					},
					fail() { // 授权失败
						uni.getSetting({
							// 获取用户的当前设置，返回值中只会出现小程序已经向用户请求过的权限
							success(res) {
								// 成功调用授权窗口
								let statu = res.authSetting
								if (!statu['scope.userLocation']) {
									// 如果设置中没有位置权限
									uni.showModal({
										// 弹窗提示
										title: '是否授权当前位置',
										content: '需要获取您的地理位置，请确认授权，否则社区功能将无法使用',
										success(tip) {
											if (tip.confirm) {
												uni.openSetting({
													// 点击确定则调其用户设置
													success(data) {
														if (data
															.authSetting[
																'scope.userLocation'
															] === true
														) {
															// 如果设置成功
															uni.showToast({
																// 弹窗提示
																title: '授权成功',
																icon: 'success',
																duration: 1000
															})
															uni.getLocation({
																// 通过getLocation方法获取数据
																type: 'gcj02',
																isHighAccuracy: true,
																success(
																	gps
																) {
																	// 成功的执行方法
																	commit
																		('setGps',
																			gps
																		)
																	if (type ===
																		0
																	) {
																		dispatch
																			(
																				'getCommunity'
																				)
																	}
																	resolve
																		(
																			gps
																			)
																}
															})
														}
													}
												})
											} else {
												if (type === 0) {
													uni.showModal({
														title: '温馨提示',
														content: '未获取到您当前位置的社区信息',
														confirmText: '选择社区',
														success: (res) => {
															if (res
																.confirm
															) {
																uni.navigateTo({
																	url: '/pages/select-community/main'
																})
															} else {}
														}
													})
												} else {
													resolve('')
												}
											}
										}
									})
								}
							}
						})
					}
				})
			})
		},
		getCommunity({
			state,
			commit,
			dispatch
		}) { // 获取城市信息
			const {
				latitude,
				longitude
			} = state.gps
			if (latitude && longitude) {
				const location = latitude + ',' + longitude
				api.commonApi.getCityInfo(location).then(res => {
					let selectCity = res.data.find(item => item.isSelected)
					// console.log(selectCity)
					commit('setCity', selectCity)
					storage.set('cityList', res.data)
					bus.$emit('getCity', res.data)
				})
			}
		},
		getButtons({commit}) {
			api.commonApi.getButtons().then(res => {
				let result = []
				function getCode(list) {
					list.forEach(ele => {
						if (typeof(ele) === 'object') {
							const chiildren = ele.children
							const code = ele.code
							if (chiildren) {
								getCode(chiildren)
							} else {
								result.push(code)
							}
						}
					})
				}
				getCode(res.data)
				let obj = {}
				result.forEach(ele => {
					obj[ele] = true
				})
				commit('setPermission', obj)
			})
		}
	}
}

export default common
