// 对uni-API进行简单封装
const toast = (title = '提示', icon = 'none', duration = 2000, mask = false) => {
	return new Promise((resolve, reject) => {
		uni.showToast({
			title,
			icon,
			duration,
			mask,
			success: res => {
				setTimeout(function() {
					resolve(res)
				}, duration)
			},
			fail: err => reject(err)
		})
	})
}

const indicator = {
	open(title = '加载中', mask = true) {
		return new Promise((resolve, reject) => {
			uni.showLoading({
				title,
				mask,
				success: res => resolve(res),
				fail: err => reject(err)
			})
		})
	},
	close() {
		uni.hideLoading()
	}
}

const modal = (options) => {
	uni.showModal(options)
}

const storage = {
	get(key) {
		return uni.getStorageSync(key)
	},
	set(key, value) {
		uni.setStorageSync(key, value)
	},
	remove(key) {
		uni.removeStorageSync(key)
	},
	clear() {
		uni.clearStorage()
	}
}

const navigator = {
	urlCache: [],
	clearUrlCache() {
		this.urlCache = []
	},
	push(url) {
		if (this.urlCache[0] === url && (url === '/pages/login/index' || url === '/pages/register/index')) {
			return
		}
		return new Promise((resolve, reject) => {
			this.urlCache.shift()
			this.urlCache.push(url)
			uni.navigateTo({
				url,
				success: res => resolve(res),
				fail: err => reject(err)
			})
		})
	},
	replace(url) {
		return new Promise((resolve, reject) => {
			uni.redirectTo({
				url,
				success: res => resolve(res),
				fail: err => reject(err)
			})
		})
	},
	back(delta) {
		return new Promise((resolve, reject) => {
			uni.navigateBack({
				delta
			})
		})
	},
	relaunch(url) {
		return new Promise((resolve, reject) => {
			uni.reLaunch({
				url,
				success: res => resolve(res),
				fail: err => reject(err)
			})
		})
	},
	switchTab(url) {
		return new Promise((resolve, reject) => {
			uni.switchTab({
				url,
				success: res => resolve(res),
				fail: err => reject(err)
			})
		})
	},
	login() {
		return this.relaunch('/pages/login/index')
	},
	// securityLogin () {
	//   return this.push('/pages/splitpackage/pages/security/main')
	// }
}
const getCurrentPosition = (type = 'wgs84') => {
	return new Promise((resolve, reject) => {
		uni.getLocation({
			type,
			success: res => resolve(res),
			fail: async err => {
				await locationErrHandler(err)
				reject(err)
			}
		})
	})
}
const chooseLocation = () => {
	return new Promise((resolve, reject) => {
		uni.chooseLocation({
			success: res => resolve(res),
			fail: async err => {
				await locationErrHandler(err)
				reject(err)
			}
		})
	})
}
const locationErrHandler = async (res) => {
	if (res.errMsg.indexOf('cancel') + 1) {
		return
	}
	if (res.errMsg.indexOf('system') + 1 || res.errMsg.indexOf('fail 1') + 1) {
		return toast('请确保开启微信定位服务')
	}
	// ios 拒绝定位 "getLocation:fail auth deny"
	modal({
		title: '提示',
		content: '需要开启【地理位置】',
		success: (res) => {
			if (res.confirm) {
				navigator.push('/pages/splitpackage/pages/member/config/main')
			} else {
				toast('取消定位')
			}
		}
	})
}
const makePhone = (phone = '') => { // 拨打电话
	return new Promise((resolve, reject) => {
		uni.makePhoneCall({
			phoneNumber: phone
		})
	})
}

const scrollTo = (location = 0, time = 700) => { // 滚动条滚动到指定位置
	return new Promise((resolve, reject) => {
		uni.pageScrollTo({
			scrollTop: location,
			duration: time
		})
		setTimeout(() => {
			resolve()
		}, time)
	})
}
const actionSheet = (actions) => {
	return new Promise((resolve, reject) => {
		uni.showActionSheet({
			itemList: actions,
			success: function(res) {
				resolve(res.tapIndex)
			},
			fail: function() {
				reject()
			}
		})
	})
}


/**
 * html 解码
 * @param str
 * @returns {string}
 */
function htmlDecode(str) {
	let s = ''
	if (!str.length) return ''
	s = str.replace(/&amp;/g, '&')
	s = s.replace(/&lt;/g, '<')
	s = s.replace(/&gt;/g, '>')
	s = s.replace(/&nbsp;/g, ' ')
	s = s.replace(/&#39;/g, '\'')
	s = s.replace(/&quot;/g, '"')

	const keyword1 = '<img'
	const keyword2 = '<p'
	let type = s.indexOf(keyword1) > -1 || s.indexOf(keyword2) > -1
	if (type) {
		s = s.replace(new RegExp(keyword1, 'gi'), `${keyword1} class="rich-img"`)
		s = s.replace(new RegExp(keyword2, 'gi'), `${keyword2} class="p"`)
	}
	return s
}

function arrayUnique(array) {
	let temp
	let arrVal
	let arrClone = array.concat() // 克隆数组
	let typeArr = { // 数组原型
		'obj': '[object Object]',
		'fun': '[object Function]',
		'arr': '[object Array]',
		'num': '[object Number]'
	}
	let ent = /(\u3000|\s|\t)*(\n)+(\u3000|\s|\t)*/gi // 空白字符正则

	// 把数组中的object和function转换为字符串形式
	for (let i = arrClone.length; i--;) {
		arrVal = arrClone[i]
		temp = Object.prototype.toString.call(arrVal)

		if (temp === typeArr['num'] && arrVal.toString() === 'NaN') {
			arrClone[i] = arrVal.toString()
		}

		if (temp === typeArr['obj']) {
			arrClone[i] = JSON.stringify(arrVal)
		}

		if (temp === typeArr['fun']) {
			arrClone[i] = arrVal.toString().replace(ent, '')
		}
	}

	// 去重关键步骤
	for (let i = arrClone.length; i--;) {
		arrVal = arrClone[i]
		temp = Object.prototype.toString.call(arrVal)

		if (temp === typeArr['arr']) this.arrayUnique(arrVal) // 如果数组中有数组，则递归
		if (arrClone.indexOf(arrVal) !== arrClone.lastIndexOf(arrVal)) { // 如果有重复的，则去重
			array.splice(i, 1)
			arrClone.splice(i, 1)
		} else {
			if (Object.prototype.toString.call(array[i]) !== temp) {
				// 检查现在数组和原始数组的值类型是否相同，如果不同则用原数组中的替换，原因是原数组经过了字符串变换
				arrClone[i] = array[i]
			}
		}
	}
	return arrClone
}

/**
 * 处理日期格式
 * @param str
 * @returns {string}
 */
function dateFormat(date, format) {
	format = format || 'yyyy-MM-dd hh:mm:ss'
	if (date !== 'Invalid Date') {
		let o = {
			'M+': date.getMonth() + 1, //month
			'd+': date.getDate(), //day
			'h+': date.getHours(), //hour
			'm+': date.getMinutes(), //minute
			's+': date.getSeconds(), //second
			'q+': Math.floor((date.getMonth() + 3) / 3), //quarter
			'S': date.getMilliseconds() //millisecond
		}
		if (/(y+)/.test(format)) format = format.replace(RegExp.$1,
			(date.getFullYear() + '').substr(4 - RegExp.$1.length))
		for (let k in o)
			if (new RegExp('(' + k + ')').test(format))
				format = format.replace(RegExp.$1,
					RegExp.$1.length === 1 ? o[k] :
					('00' + o[k]).substr(('' + o[k]).length))
		return format
	}
	return ''

}

/**
 * 当前时间增加分钟数
 *
 */
function dateAdd(num) {
	var dt = new Date(); //得到时间对象
	var tmNow = dt.getMinutes(); //得到当前分钟,此处只是举个例子
	var tmNowNext = tmNow + num //
	dt.setMinutes(tmNowNext, 0, 0)
	return dt
}

/**
 * 获取数组某个元素的下标
 * arrays  : 传入的数组
 * obj     : 需要获取下标的元素
 */

function contains(arrays, obj) {
	let i = arrays.length;
	while (i--) {
		if (arrays[i] === obj) {
			return i;
		}
	}
	return false;
}


/***
 * 连续输入节流阀
 * func 输入完成的回调函数
 * delay 延迟时间
 */
function debounce(func, delay) {
	let timer
	return (...args) => {
		if (timer) {
			clearTimeout(timer)
		}
		timer = setTimeout(() => {
			func.apply(this, args)
		}, delay)
	}
}



export {
	actionSheet,
	toast,
	indicator,
	modal,
	storage,
	navigator,
	makePhone,
	scrollTo,
	getCurrentPosition,
	chooseLocation,
	htmlDecode,
	arrayUnique,
	dateFormat,
	contains,
	dateAdd,
	debounce
}
